from __future__ import unicode_literals, absolute_import, print_function
import platform
import appdirs


class ContextInfo:
    @property
    def platform(self):
        return platform.system()


class ContextStore:
    pass


class Context:
    __store = ContextStore()
    info = ContextInfo()

    @property
    def store(self):
        return self.__store

    @staticmethod
    def paths_for_state(state_name, app_author="tdsc", version=None, roaming=False):
        return appdirs.AppDirs(
            state_name, appauthor=app_author, version=version, roaming=roaming
        )



from __future__ import unicode_literals, absolute_import, print_function
from abc import ABC, abstractmethod
import logging

"""
Define conventional "state" behaviour.

The simplest state is an object that encapsulates a pre-execution guard,
``should_execute``, an action to be performed, ``execute``, and a post-execute
callback and transform, ``post_execute``. By default, ``should_execute`` is
``True``, and ``post_execute`` returns the result or raised error from
``execute``, or ``None`` if ``should_execute`` guarded against execution.
"""


class State(ABC):
    def _get_logger(self):
        return logging.getLogger("tdsc.state.{}".format(self.__class__.__name__))

    def should_execute(self, ctx):
        return True

    @abstractmethod
    def execute(self, ctx):
        raise NotImplemented

    def post_execute(self, ctx, ran_successfully, result):
        if not ran_successfully:
            self._get_logger().exception(
                "State {} did not run successfully.".format(self.__class__.__name__),
                exc_info=result,
            )
        return result

    def run(self, ctx):
        result = None
        ran_successfully = False

        try:
            if self.should_execute(ctx):
                result = self.execute(ctx)
                ran_successfully = True
        except Exception as err:
            result = err
        finally:
            return self.post_execute(ctx, ran_successfully, result)



from __future__ import unicode_literals, absolute_import, print_function

from .context import Context


class Strategy:
    def __init__(self, strategies=None, states=None):
        self.strategies = strategies or []
        self.states = states or []

    def run(self, context=None):
        context = context or Context()

        for state in self.states:
            state.run(context)

        for strategy in self.strategies:
            strategy.run(context)



import subprocess
import functools


@functools.lru_cache(maxsize=1)
def git_available():
    """
    Indicates whether ``git`` is available on the system.

    Result is cached by functools.lru_cache
    """

    return (
        subprocess.call(
            ["git", "--version"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
        )
        == 0
    )


def run_git(*args, **kwargs):
    """Shallow wrapper to run ``git`` with given arguments, return exit code."""
    args = ["git"] + list(args)

    return subprocess.call(args, **kwargs)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['tdsc', 'tdsc.utils']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4,<2.0', 'distro>=1.3,<2.0', 'six>=1.11,<1.12', 'toml>=0.9,<0.10']

setup_kwargs = {
    'name': 'tdsc',
    'version': '0.1.1',
    'description': '',
    'long_description': '################################\nTiny Desired State Configuration\n################################\n\nWARNING: Until version ``0.2``, *tdsc* should be considered unusably unstable.\n\n*tdsc* is a lightweight "desired state configuration" library that aims to\nmake cross-machine configuration a pleasant experience.\n\n********\nFeatures\n********\n\n* Python support: ``2.7``, ``3.3+``\n',
    'author': 'Liam Dawson',
    'author_email': 'liam@ldaws.com',
    'url': 'https://github.com/liamdawson/tdsc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7',
}


setup(**setup_kwargs)
